//----------------------------------------------------------------------
//      AVR-GCC-специфічні макроси
// 	Корекція PROGMEM в С++ для avr-gcc
//	Інші часто використовувані макроси

#pragma once

#include "gcc_macros.h" // Незалежне від архітектури

//----------------------------------------------------------------------
//  часто використовувані атрибути (avr-gcc-специфічні)
#ifndef OS_MAIN
#define OS_MAIN __attribute__((__OS_main__))
#endif

#ifndef OS_TASK
#define OS_TASK __attribute__((__OS_task__))
#endif


//----------------------------------------------------------------------
// https://www.mikrocontroller.net/topic/65923#530326

#define PRELOAD(reg,var) \
  __asm__ __volatile (";RELOAD " reg " with " #var : "=" reg (var) : "0" (var))

#define MEM_BARRIER \
    __asm volatile ("; Barrier " ::: "memory")

#define CSE_BARRIER \
    __asm volatile ("; CSE-Barrier " ::)


//----------------------------------------------------------------------
//      Циклічні зсуви
#define rotl8(var)      \
(__extension__({        \
    uint8_t __result = (uint8_t)(var);  \
    __asm__ __volatile__ (        \
        "lsl %0"  "\n\t"          \
        "adc %0, __zero_reg__"    \
        : "+r" (__result)         \
	:                         \
    );                            \
    __result;                     \
}))

#define rotr8(var)      \
(__extension__({        \
    uint8_t __result = (uint8_t)(var);  \
    __asm__ __volatile__ (      \
        "lsr %0"    "\n\t"      \
        "brcc .+2"  "\n\t"      \
        "ori %0, 128"           \
        : "+d" (__result)       \
	:                       \
    );                          \
    __result;                   \
}))


#define rotl16(var)      \
(__extension__({         \
    uint16_t __result = (uint16_t)(var);  \
    __asm__ __volatile__ (        \
        "lsl %A0"  "\n\t"         \
        "rol %B0"  "\n\t"         \
        "adc %A0, __zero_reg__"   \
        : "+r" (__result)         \
	:                         \
    );                            \
    __result;                     \
}))

#define rotr16(var)      \
(__extension__({         \
    uint16_t __result = (uint16_t)(var);  \
    __asm__ __volatile__ (    \
        "lsr %B0"  "\n\t"     \
        "ror %A0"  "\n\t"     \
        "brcc .+2" "\n\t"     \
        "ori %B0, 128"        \
        : "+d" (__result)     \
	:                     \
    );                        \
    __result;                 \
}))




/*--  INIT(number)  -----------------------------------------------------------
 *  Макрос для задання коду в секціях .init#number
 *  Для сумісності та для зручності аналізу згенерованого компілятором коду
 *  залишено варіант з двома аргументами INIT_CODE(number,name)
 *  name - ім'я псевдо-функції, можна використовувати
 *	як коментар. В межах файлу імена мають бути унікальними.
 *
 *  Наприклад:
 *  INIT(1)
 *  {
 *    MCUCR = (1 << SRE);
 *    XMCRA = (4 << SRL0) | (0 << SRW00);
 *    XMCRB = (1 << XMBK);
 *  }
 *  або
 *  INIT_CODE(1,setup_extram)
 *  {
 *    MCUCR = (1 << SRE);
 *    XMCRA = (4 << SRL0) | (0 << SRW00);
 *    XMCRB = (1 << XMBK);
 *  }
 *
 *  розподіл секцій для avr-gcc:
 *  .init0  Start here after reset.
 *  .init1
 *  .init2  Clear __zero_reg__, set up stack pointer.
 *  .init3
 *  .init4  Initialize data and BSS.
 *  .init5
 *  .init6  C++ constructors.
 *  .init7
 *  .init8
 *  .init9  Call main().
 *
 */


#define INIT_CODE(_number_,_name_)\
    static void init_##_number_##_##_name_ (void) __attribute__ ((section(".init" #_number_), naked, used));\
    static void init_##_number_##_##_name_ (void)

#define INIT_CODE_(n,l) INIT_CODE(n,l)
#define INIT(_number_) INIT_CODE_(_number_,__LINE__)

/*------        end of file  avrgcc_macros.h        ------*/

